def str_fsize(sz):
    """
    Formats file size as string (i.e., 1.2 Mb)
    """
    if sz < 1024:
        return f'{sz:.1f} bytes'
    sz /= 1024.0
    if sz < 1024:
        return f'{sz:.1f} KB'
    sz /= 1024.0
    if sz < 1024:
        return f'{sz:.1f} MB'
    sz /= 1024.0
    if sz < 1024:
        return f'{sz:.1f} GB'
    sz /= 1024.0
    return f'{sz:.1f} TB'





import daemon
import logging
import traceback
from datetime import datetime

from aj.util.pidfile import PidFile


def start(daemonize=False, log_level=logging.INFO, dev_mode=False, **kwargs):
    """
    A wrapper for :func:`run` that optionally runs it in a forked daemon process.

    :param kwargs: rest of arguments is forwarded to :func:`run`
    """
    # reimport into scope
    import aj

    if daemonize:
        context = daemon.DaemonContext(
            pidfile=PidFile('/var/run/ajenti.pid'),
            detach_process=True,
            files_preserve=list(range(1024)),  # force-closing files breaks gevent badly
        )
        with context:
            aj.log.init_log_directory()
            aj.log.init_log_file(log_level)
            import aj.core
            try:
                aj.core.run(dev_mode=dev_mode, **kwargs)
            # pylint: disable=W0703
            except Exception as e:
                handle_crash(e)
    else:
        if not dev_mode:
            aj.log.init_log_directory()
            aj.log.init_log_file(log_level)
        import aj.core
        try:
            aj.core.run(dev_mode=dev_mode, **kwargs)
        except KeyboardInterrupt:
            pass
        # pylint: disable=W0703
        except Exception as e:
            handle_crash(e)


def handle_crash(exc):
    # todo rework this
    now = datetime.now().strftime('%Y-%m-%d-%Hh%M')
    logging.error('Fatal crash occured')
    traceback.print_exc()
    exc.traceback = traceback.format_exc()
    report_path = f'/var/log/ajenti/crash-{now}.txt'
    try:
        report = open(report_path, 'w')
    except Exception as e:
        report_path = f'./crash-{now}.txt'
        report = open(report_path, 'w')

    from aj.util import make_report
    report.write(make_report(exc))
    report.close()
    logging.error(f'Crash report written to {report_path}')
    logging.error('Please submit it to https://github.com/ajenti/ajenti/issues/new')



from gevent import pywsgi, spawn

def http_to_https(target_url):

    def app(environ, start_response):

        if environ['PATH_INFO'] == "/":
            start_response(
                            '301 Move Permanently',
                            [
                                ('Location', target_url),
                                ('Connection', 'close'),
                                ('Cache-control', 'private')
                            ])
            return [b"""
    <html>
    <body>
        Migrating to HTTPS protocol.
        Please go to <a href='{self.target}'>{self.target}</a> to use Ajenti.
    </body>
    </html>"""]
        start_response('404 Not Found', [('Content-Type', 'text/html')])

        return [b'<h1>Not Found</h1>']

    pywsgi.WSGIServer(
        ('', 80),
        application = app,
    ).serve_forever()



#!/usr/bin/env python3
from setuptools import setup
from setuptools.command.install import install as _install
from setuptools import find_packages

import os
import socket
import subprocess
import sys


def _post_install(scripts_dir):
    config_path = '/etc/ajenti/config.yml'
    users_file = '/etc/ajenti/users.yml'

    if not os.path.exists('/etc/ajenti'):
        os.makedirs('/etc/ajenti')
    if not os.path.exists(config_path):
        with open(config_path, 'w') as c:
            c.write(f'''
auth:
  allow_sudo: true
  emails: {{}}
  provider: os
  users_file: {users_file}
bind:
  host: 0.0.0.0
  mode: tcp
  port: 8000
color: default
max_sessions: 9
trusted_domains: []
trusted_proxies: []
session_max_time: 3600
name: {socket.gethostname()}
ssl:
  certificate:
  fqdn_certificate:
  force: false
  client_auth:
    certificates: []
    enable: false
    force: false
  enable: false
        ''')

    if not os.path.exists(users_file):
        with open(users_file, 'w') as u:
            u.write("users: null")
    subprocess.call([sys.executable, 'ajenti-ssl-gen', socket.gethostname()], cwd=scripts_dir)


class install(_install):
    def run(self):
        _install.run(self)
        self.execute(_post_install, [self.install_scripts], msg='Running post install script')


setup(
    name='ajenti-panel',
    version='2.2.1',
    python_requires='>=3',
    install_requires=[
        'aj==2.2.1',
        'pyyaml',
        'requests',
    ],
    description='Ajenti core based panel',
    author='Eugene Pankov',
    author_email='e@ajenti.org',
    url='https://ajenti.org/',
    packages=find_packages(),
    package_data={
        "": [
            "static/images/error.jpeg",
            "static/images/Logo.png",
            "static/emails/reset_email.html",
        ]},
    scripts=['ajenti-panel', 'ajenti-ssl-gen', 'ajenti-client-ssl-gen', 'ajenti-upgrade'],
    cmdclass={'install': install},
)
